FazBrowse GitHub Viewer
|
Trending
|
URL:
|
Home
Tools:
[Download Repo ZIP]
[View Raw Code]
[Original HTTPS Page]
CaffeOnACL/src/caffe/layers/base_data_layer.cpp at master · 2php/CaffeOnACL · GitHub
2php
CaffeOnACL
Repository navigation
Code
Pull requests
Actions
Projects
Wiki
Security and quality
Insights
Expand file tree
Breadcrumbs
CaffeOnACL
/
src
/
caffe
/
layers
/
base_data_layer.cpp
Copy path
More file actions
More file actions
Latest commit
History
History
History
136 lines (123 loc) · 4.02 KB
Breadcrumbs
CaffeOnACL
/
src
/
caffe
/
layers
/
base_data_layer.cpp
Copy path
File metadata and controls
136 lines (123 loc) · 4.02 KB
Raw
Copy raw file
Download raw file
Open symbols panel
Edit and raw actions
1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
#
include
<
boost/thread.hpp
>
#
include
<
vector
>
#
include
"
caffe/blob.hpp
"
#
include
"
caffe/data_transformer.hpp
"
#
include
"
caffe/internal_thread.hpp
"
#
include
"
caffe/layer.hpp
"
#
include
"
caffe/layers/base_data_layer.hpp
"
#
include
"
caffe/proto/caffe.pb.h
"
#
include
"
caffe/util/blocking_queue.hpp
"
namespace
caffe
{
template
<
typename
Dtype>
BaseDataLayer<Dtype>::BaseDataLayer(
const
LayerParameter& param)
: Layer<Dtype>(param),
transform_param_
(param.transform_param()) {
}
template
<
typename
Dtype>
void
BaseDataLayer<Dtype>::LayerSetUp(
const
vector<Blob<Dtype>*>& bottom,
const
vector<Blob<Dtype>*>& top) {
if
(top.
size
() ==
1
) {
output_labels_ =
false
;
}
else
{
output_labels_ =
true
;
}
data_transformer_.
reset
(
new
DataTransformer<Dtype>(transform_param_,
this
->
phase_
));
data_transformer_->
InitRand
();
//
The subclasses should setup the size of bottom and top
DataLayerSetUp
(bottom, top);
}
template
<
typename
Dtype>
BasePrefetchingDataLayer<Dtype>::BasePrefetchingDataLayer(
const
LayerParameter& param)
: BaseDataLayer<Dtype>(param),
prefetch_
(param.data_param().prefetch()),
prefetch_free_(), prefetch_full_(), prefetch_current_() {
for
(
int
i =
0
; i < prefetch_.
size
(); ++i) {
prefetch_[i].
reset
(
new
Batch<Dtype>());
prefetch_free_.
push
(prefetch_[i].
get
());
}
}
template
<
typename
Dtype>
void
BasePrefetchingDataLayer<Dtype>::LayerSetUp(
const
vector<Blob<Dtype>*>& bottom,
const
vector<Blob<Dtype>*>& top) {
BaseDataLayer<Dtype>::
LayerSetUp
(bottom, top);
//
Before starting the prefetch thread, we make cpu_data and gpu_data
//
calls so that the prefetch thread does not accidentally make simultaneous
//
cudaMalloc calls when the main thread is running. In some GPUs this
//
seems to cause failures if we do not so.
for
(
int
i =
0
; i < prefetch_.
size
(); ++i) {
prefetch_[i]->
data_
.
mutable_cpu_data
();
if
(
this
->
output_labels_
) {
prefetch_[i]->
label_
.
mutable_cpu_data
();
}
}
#
ifndef
CPU_ONLY
if
(
Caffe::mode
() == Caffe::
GPU
) {
for
(
int
i =
0
; i < prefetch_.
size
(); ++i) {
prefetch_[i]->
data_
.
mutable_gpu_data
();
if
(
this
->
output_labels_
) {
prefetch_[i]->
label_
.
mutable_gpu_data
();
}
}
}
#
endif
DLOG
(
INFO
) <<
"
Initializing prefetch
"
;
this
->
data_transformer_
->
InitRand
();
StartInternalThread
();
DLOG
(
INFO
) <<
"
Prefetch initialized.
"
;
}
template
<
typename
Dtype>
void
BasePrefetchingDataLayer<Dtype>::InternalThreadEntry() {
#
ifndef
CPU_ONLY
cudaStream_t stream;
if
(
Caffe::mode
() == Caffe::
GPU
) {
CUDA_CHECK
(
cudaStreamCreateWithFlags
(&stream, cudaStreamNonBlocking));
}
#
endif
try
{
while
(!
must_stop
()) {
Batch<Dtype>* batch = prefetch_free_.
pop
();
load_batch
(batch);
#
ifndef
CPU_ONLY
if
(
Caffe::mode
() == Caffe::
GPU
) {
batch->
data_
.
data
().
get
()->
async_gpu_push
(stream);
if
(
this
->
output_labels_
) {
batch->
label_
.
data
().
get
()->
async_gpu_push
(stream);
}
CUDA_CHECK
(
cudaStreamSynchronize
(stream));
}
#
endif
prefetch_full_.
push
(batch);
}
}
catch
(boost::thread_interrupted&) {
//
Interrupted exception is expected on shutdown
}
#
ifndef
CPU_ONLY
if
(
Caffe::mode
() == Caffe::
GPU
) {
CUDA_CHECK
(
cudaStreamDestroy
(stream));
}
#
endif
}
template
<
typename
Dtype>
void
BasePrefetchingDataLayer<Dtype>::Forward_cpu(
const
vector<Blob<Dtype>*>& bottom,
const
vector<Blob<Dtype>*>& top) {
if
(prefetch_current_) {
prefetch_free_.
push
(prefetch_current_);
}
prefetch_current_ = prefetch_full_.
pop
(
"
Waiting for data
"
);
//
Reshape to loaded data.
top[
0
]->
ReshapeLike
(prefetch_current_->
data_
);
top[
0
]->
set_cpu_data
(prefetch_current_->
data_
.
mutable_cpu_data
());
if
(
this
->
output_labels_
) {
//
Reshape to loaded labels.
top[
1
]->
ReshapeLike
(prefetch_current_->
label_
);
top[
1
]->
set_cpu_data
(prefetch_current_->
label_
.
mutable_cpu_data
());
}
}
#
ifdef
CPU_ONLY
STUB_GPU_FORWARD
(BasePrefetchingDataLayer, Forward);
#
endif
INSTANTIATE_CLASS
(BaseDataLayer);
INSTANTIATE_CLASS
(BasePrefetchingDataLayer);
}
//
namespace caffe
Back
|
FazBrowse Home
|
New Git URL